@@ -58,6 +58,7 @@
 #include "XrdPosix/XrdPosixExtra.hh"
 #include "XrdPosix/XrdPosixInfo.hh"
 #include "XrdPosix/XrdPosixXrootd.hh"
+#include "XrdOfs/XrdOfsFSctl_PI.hh"
 
 #include "XrdOss/XrdOssError.hh"
 #include "XrdOuc/XrdOucEnv.hh"
@@ -65,6 +66,7 @@
 #include "XrdOuc/XrdOucPgrwUtils.hh"
 #include "XrdSec/XrdSecEntity.hh"
 #include "XrdSecsss/XrdSecsssID.hh"
+#include "XrdSfs/XrdSfsInterface.hh"
 #include "XrdSys/XrdSysError.hh"
 #include "XrdSys/XrdSysHeaders.hh"
 #include "XrdSys/XrdSysPlatform.hh"
@@ -93,8 +95,10 @@ class XrdScheduler;
 
 namespace XrdProxy
 {
+thread_local XrdOucECMsg ecMsg("[pss]");
+
 static XrdPssSys   XrdProxySS;
-  
+
        XrdSysError eDest(0, "pss_");
 
        XrdScheduler *schedP = 0;
@@ -103,6 +107,8 @@ static XrdPssSys   XrdProxySS;
 
        XrdOucEnv    *envP   = 0;
 
+       XrdOfsFSctl_PI *cacheFSctl = nullptr;
+
        XrdSecsssID  *idMapper = 0;    // -> Auth ID mapper
 
 static const char   *ofslclCGI = "ofs.lcl=1";
@@ -187,6 +193,12 @@ int XrdPssSys::Init(XrdSysLogger *lp, const char *cFN, XrdOucEnv *envP)
    tmp = ((NoGo = Configure(cFN, envP)) ? "failed." : "completed.");
    eDest.Say("------ Proxy storage system initialization ", tmp);
 
+// Extract Pfc control, if it is there.
+//
+  if (!NoGo)
+      cacheFSctl = (XrdOfsFSctl_PI*)envP->GetPtr("XrdFSCtl_PC*");
+
+
 // All done.
 //
    return NoGo;
@@ -355,7 +367,7 @@ int XrdPssSys::Mkdir(const char *path, mode_t mode, int mkpath, XrdOucEnv *eP)
 
 // Simply return the proxied result here
 //
-   return (XrdPosixXrootd::Mkdir(pbuff, mode) ? -errno : XrdOssOK);
+   return (XrdPosixXrootd::Mkdir(pbuff, mode) ? Info(errno) : XrdOssOK);
 }
   
 /******************************************************************************/
@@ -767,6 +779,28 @@ int XrdPssFile::Open(const char *path, int Oflag, mode_t Mode, XrdOucEnv &Env)
           }
       }
 
+   // check CGI cache-control paramters
+   if (cacheFSctl)
+   {
+      int elen;
+      char *envcgi = (char *)Env.Env(elen);
+
+      if (envcgi && strstr(envcgi, "only-if-cached"))
+      {
+         XrdOucErrInfo einfo;
+         XrdSfsFSctl myData;
+         myData.Arg1 = "cached";
+         myData.Arg1Len = 1;
+         myData.Arg2Len = 1;
+         const char *myArgs[1];
+         myArgs[0] = path;
+         myData.ArgP = myArgs;
+         int fsctlRes = cacheFSctl->FSctl(SFS_FSCTL_PLUGXC, myData, einfo);
+         if (fsctlRes == SFS_ERROR)
+            return -einfo.getErrInfo();
+      }
+   }
+
 // If this is a third party copy open, then strange rules apply. If this is an
 // outgoing proxy we let everything pass through as this may be a TPC request
 // elsewhere.  Otherwise, if it's an open for reading, we open the file but
@@ -1234,6 +1268,16 @@ int XrdPssFile::Ftruncate(unsigned long long flen)
     return (XrdPosixXrootd::Ftruncate(fd, flen) ?  -errno : XrdOssOK);
 }
   
+/******************************************************************************/
+/*                      I n t e r n a l   M e t h o d s                       */
+/******************************************************************************/
+
+int XrdPssSys::Info(int rc)
+{
+   XrdPosixXrootd::QueryError(XrdProxy::ecMsg.Msg());
+   return -rc;
+}
+  
 /******************************************************************************/
 /*                                 P 2 D S T                                  */
 /******************************************************************************/
